@@ -24,16 +24,13 @@ from uuid import UUID
 
 from eventsourcing.application.process import ProcessApplication
 from eventsourcing.application.sqlalchemy import SQLAlchemyApplication
-from eventsourcing.system.runner import (
-    InProcessRunner,
-    SingleThreadedRunner,
-)
-from eventsourcing.system.definition import System
 from eventsourcing.domain.model.aggregate import (
     AggregateRoot,
     TAggregate,
     TAggregateEvent,
 )
+from eventsourcing.system.definition import System
+from eventsourcing.system.runner import InProcessRunner, SingleThreadedRunner
 
 
 # Locations in the world.
